@@ -14,16 +14,17 @@ _FP32_OPS: Set[str] = {
     # math with steep/ill-conditioned curves
     "log", "exp", "sqrt", "cbrt", "log10", "log2",
     "sin", "cos", "tan",
-    # softmax & reductions & batchnormalizer that accumulate / are variance-based
+    # softmax & reductions that accumulate / are variance-based
     "softmax",
     "sum", "mean", "std", "var",
-    # "batchnormalizer",
     # losses (compute entirely in fp32)
     "mse", "rmse", "mae", "binarycrossentropy", "categoricalcrossentropy",
     # casting decisions shouldn’t be overridden by autocast
     "cast",
     # pow is risky in fp16 except for tiny integer exponents
     "pow",
+    # normalization
+    "batchnormalizer",
 }
 
 # Ops that are **safe** to run in FP16 (or BF16) by default
@@ -31,7 +32,7 @@ _FP16_SAFE_OPS: Set[str] = {
     # arithmetic
     "add", "sub", "mul", "div",
     # linalg (prefer fp32 accumulation inside the kernel)
-    "matmul", "dot", "linear", "tensordot", "einsum", "transpose",
+    "matmul", "tensordot", "einsum", "transpose",
     # tensor reshapes / views
     "reshape", "flatten", "squeeze", "expanddims", "slidingwindowview",
     # padding and elementwise
@@ -42,6 +43,9 @@ _FP16_SAFE_OPS: Set[str] = {
 
 
 
+# FP32 ops that should not pre-cast inputs; instead, compute with FP32 accumulation
+_FP32_NO_PRECAST: Set[str] = {"sum", "mean", "std", "var", "batchnormalizer"}
+
 def should_cast_to_fp16(op_name: str) -> bool:
     """
     Determine if an operation should be cast to FP16 in autocast context.
@@ -98,6 +102,9 @@ def maybe_cast_tensor(tensor, target_dtype=None, op_name: str = "unknown") -> 'T
         if should_cast_to_fp16(op_name):
             target_dtype = autocast.get_autocast_dtype()
         else:
+            # Avoid pre-casting large tensors for certain FP32 ops; rely on op to use FP32 accumulate
+            if op_name and op_name.lower() in _FP32_NO_PRECAST:
+                return tensor
             return tensor.cast(ng.float32)
     
     # Only cast if different from current dtype
