@@ -171,7 +171,7 @@ class Pad(Function, Module):
         if not hasattr(self, "pad_width"):
             raise RuntimeError("pad_width not set; call forward() first.")
         del parent_tensors[0].data  # free parent tensor data to save memory
-        # ng.flush(gc=False)
+        ng.flush(gc=False)
         slices = tuple(slice(l, None if u == 0 else -u) for (l, u) in self.pad_width)
         parent_tensors[0].data = output_tensor.data[slices]
 
@@ -195,51 +195,44 @@ class Clone(Function, Module):
 
 
 class SlidingWindowView(Function, Module):
-    """Compact sliding window view using as_strided for all cases."""
+    """
+    Smart Vectorized Sliding Window View with AutoDiff Support using as_strided.
+    """
     def __init__(self, window_shape: Sequence[int],
                  axes: Union[int, Tuple[int, ...]] = (2, 3),
                  strides: Union[int, Tuple[int, ...]] = (1, 1)):
         Function.__init__(self)
         Module.__init__(self)
         self.axes = axes if isinstance(axes, tuple) else (axes,)
-        self.strides = strides if isinstance(strides, tuple) else tuple(strides for _ in range(len(self.axes)))
-        self.window_shape = window_shape if isinstance(window_shape, tuple) else tuple(window_shape for _ in range(len(self.axes)))
-    
+        self.strides = strides if isinstance(strides, tuple) else \
+                       tuple(strides for _ in range(len(self.axes)))
+        self.window_shape = window_shape if isinstance(window_shape, tuple) else \
+                           tuple(window_shape for _ in range(len(axes)))
     def forward(self, A: xp.ndarray) -> xp.ndarray:
-        # Calculate output shape
-        out_shape = list(A.shape)
-        for i, (ax, ws, st) in enumerate(zip(self.axes, self.window_shape, self.strides)):
-            out_shape[ax] = (A.shape[ax] - ws) // st + 1
-        out_shape.extend(self.window_shape)
-        
-        # Calculate strides
-        out_strides = list(A.strides)
-        for ax, st in zip(self.axes, self.strides):
-            out_strides[ax] *= st
-        out_strides.extend([A.strides[ax] for ax in self.axes])
-        
-        return xp.lib.stride_tricks.as_strided(A, shape=tuple(out_shape), strides=tuple(out_strides))
-    
+        # Compute output shape and strides
+        new_shape = list(A.shape)
+        new_strides = list(A.strides)
+        for i, ax in enumerate(self.axes):
+            new_shape[ax] = (A.shape[ax] - self.window_shape[i]) // self.strides[i] + 1
+            new_strides[ax] = A.strides[ax] * self.strides[i]
+        # Append window dimensions
+        for i, ax in enumerate(self.axes):
+            new_shape.append(self.window_shape[i])
+            new_strides.append(A.strides[ax])
+        # Create strided view
+        self.forward_shape = tuple(new_shape)
+        self.forward_strides = tuple(new_strides)
+        return xp.lib.stride_tricks.as_strided(
+            A, shape=self.forward_shape, strides=self.forward_strides
+        )
     def backward(self, grad_output: xp.ndarray) -> xp.ndarray:
         A = self.parent_tensors[0]
-        if not A.requires_grad:
-            return None
-        
         grad_buffer = xp.zeros(A.shape, dtype=grad_output.dtype)
-        kwargs = {"writeable": True} if xp is np else {}
-        
-        # Create gradient view with same logic as forward
-        out_shape = list(grad_buffer.shape)
-        for ax, ws, st in zip(self.axes, self.window_shape, self.strides):
-            out_shape[ax] = (grad_buffer.shape[ax] - ws) // st + 1
-        out_shape.extend(self.window_shape)
-        
-        out_strides = list(grad_buffer.strides)
-        for ax, st in zip(self.axes, self.strides):
-            out_strides[ax] *= st
-        out_strides.extend([grad_buffer.strides[ax] for ax in self.axes])
-        
-        grad_view = xp.lib.stride_tricks.as_strided(grad_buffer, shape=tuple(out_shape), strides=tuple(out_strides), **kwargs)
+        # Create strided view for gradient accumulation
+        grad_view = xp.lib.stride_tricks.as_strided(
+            grad_buffer, shape=self.forward_shape, strides=self.forward_strides
+        )
+        # Accumulate gradients
         grad_view += grad_output
         return grad_buffer
     
